package category

import (
	"context"
	"fmt"

	"github.com/jackc/pgx/v5"
	"github.com/jackc/pgx/v5/pgxpool"
)

type CategoryEntity struct {
	Id           int
	Name         string
	UserId       int
	Favorite     bool
	MonthlyLimit int
}

type CategoryService struct {
	db *pgxpool.Pool
}

func NewCategoryService(
	db *pgxpool.Pool,
) *CategoryService {
	return &CategoryService{
		db: db,
	}
}

func (s *CategoryService) AddCategory(ctx context.Context, category *CategoryEntity) (*CategoryEntity, error) {
	query := `INSERT INTO categories (name, user_id, favorite, monthly_limit) VALUES (@name, @user_id, @favorite, @monthly_limit)`
	args := pgx.NamedArgs{
		"name":          category.Name,
		"user_id":       1,
		"favorite":      category.Favorite,
		"monthly_limit": category.MonthlyLimit,
	}
	_, err := s.db.Exec(ctx, query, args)
	if err != nil {
		return nil, fmt.Errorf("unable to insert row: %w", err)
	}
	return category, nil
}
