package user

import (
	"context"
	"errors"
	"fmt"

	"github.com/jackc/pgx/v5"
	"github.com/jackc/pgx/v5/pgconn"
	"github.com/jackc/pgx/v5/pgxpool"
)

type UserEntity struct {
	Id       int
	Username string
}

type UsernameAlreadyExistsErr struct{}

func (e *UsernameAlreadyExistsErr) Error() string {
	return "username already exists error"
}

type UserService struct {
	db *pgxpool.Pool
}

func NewUserService(
	db *pgxpool.Pool,
) *UserService {
	return &UserService{
		db: db,
	}
}

func (s *UserService) AddUser(ctx context.Context, user *UserEntity) (*UserEntity, error) {
	query := `INSERT INTO users (username) VALUES (@username) RETURNING id`
	args := pgx.NamedArgs{
		"username": user.Username,
	}
	if err := s.db.QueryRow(ctx, query, args).Scan(&user.Id); err != nil {
		var pgErr *pgconn.PgError
		if errors.As(err, &pgErr) {
			if pgErr.Code == "23505" && pgErr.ConstraintName == "users_username_key" { // unique_violation
				return nil, &UsernameAlreadyExistsErr{}
			}
		}
		return nil, fmt.Errorf("unable to insert row: %w", err)
	}
	return user, nil
}
